# -*- coding: utf-8 -*-
"""
This gives access to the pasteboard.
"""

from rubicon.objc import *

__UIPasteboard__ = ObjCClass("UIPasteboard")

def generalPasteboard():
    raise NameError("`generalPasteboard` was renamed to `general_pasteboard`")

def setString(text):
    raise NameError("`setString` was renamed to `set_string`")

def setStrings(array):
    raise NameError("`setStrings` was renamed to `set_strings`")

def setImage(image):
    raise NameError("`setImage` was renamed to `set_image`")

def setImages(array):
    raise NameError("`setImages` was renamed to `set_images`")

def setURL(url):
    raise NameError("`setURL` was renamed to `set_url`")

def setURLs(array):
    raise NameError("`setURLs` was renamed to `set_urls`")

def general_pasteboard():
    """
    Returns the Objective-C representation of the general pasteboard.
    """
    return __UIPasteboard__.generalPasteboard

# MARK: - Text

def string():
    """
    Returns the text contained in the pasteboard.
    """
    return str(generalPasteboard().string)

def strings():
    """
    Returns all strings contained in the pasteboard.
    """
    return generalPasteboard().strings

def set_string(text):
    """
    Copy the given text to the pasteboard.
    
    Args:
        text: The text to copy.
    """
    generalPasteboard().string = text

def set_strings(array):
    """
    Copy the given strings to the pasteboard.
        
    Args:
        array: A list of strings to copy.
    """
    generalPasteboard().strings = array

# MARK: - Images

def image():
    """
    Returns the image contained in the pasteboard as an Objective-C `UIImage`.
    """
    return generalPasteboard().image

def images():
    """
    Returns all images contained in the pasteboard as Objective-C `UIImage`s.
    """
    return generalPasteboard().image

def set_image(image):
    """
    Copy the given image to the pasteboard.
        
    Args:
        image: The image to copy.
    """
    generalPasteboard().image = text

def set_images(array):
    """
    Copy the given images to the pasteboard.
        
    Args:
        array: A list of images to copy.
    """
    generalPasteboard().images = array

# MARK: - URLs

def url():
    """
    Returns the URL contained in the pasteboard as an Objective-C `NSURL`.
    """
    return generalPasteboard().url

def urls():
    """
    Returns all URLs contained in the pasteboard as Objective-C `NSURL`s.
    """
    return generalPasteboard().urls

def set_url(url):
    """
    Copy the given URL to the pasteboard.
        
    Args:
        url: The Objective-C `NSURL` to copy.
    """
    generalPasteboard().url = url

def set_urls(array):
    """
    Copy the given URLs to the pasteboard.
        
    Args:
        array: A list of Objective-C `NSURL`s to copy.
    """
    generalPasteboard().urls = array
