# -*- coding: utf-8 -*-
"""
This module allows you to share items, to import files and to open URLs.
"""

import pyto
import mainthread
from rubicon.objc import *

__PySharingHelper__ = pyto.PySharingHelper

def shareItems(items):
    raise NameError("`shareItems` was renamed to `share_items`")

def pickDocumentsWithFilePicker(items):
    raise NameError("`pickDocumentsWithFilePicker` was renamed to `pick_documents`")

def pickedFiles():
    raise NameError("`pickedFiles` was renamed to `picked_files`")

def openURL(url):
    raise NameError("`openURL` was renamed to `open_url`")

def share_items(items):
    """
    Opens a share sheet with given items.
        
    Args:
        items: Items to be shared with the sheet.
    """
    __PySharingHelper__.share(items)

# MARK: - File picker

FilePicker = pyto.FilePicker
"""
A class representing a file picker.
    
Example:
    filePicker = sharing.FilePicker.new()
    filePicker.fileTypes = ["public.data"]
    filePicker.allowsMultipleSelection = True
    
    def filesPicked() -> None:
        files = sharing.pickedFiles()
        sharing.share_items(files)
    
    filePicker.completion = filesPicked
    sharing.pick_documents(filePicker)
    """

def pick_documents(filePicker):
    """
    Pick documents with given parameters as a `FilePicker`.
        
    Args:
        filePicker: The parameters of the file picker to be presented.
    """
    __PySharingHelper__.presentFilePicker(filePicker)

def picked_files():
    """
    Returns files picked with `pickDocumentsWithFilePicker` as URLs.
    """
    return FilePicker.urls

# MARK: - URLs

__NSURL__ = ObjCClass("NSURL")
__UIApplication__ = ObjCClass("UIApplication")

def open_url(url):
    """
    Opens the given URL.
    
    Args:
        url: URL to open. Can be a String or an Objective-C `NSURL`.
    """
    
    def __openURL__() -> None:
        __UIApplication__.sharedApplication.openURL(__url__)
    
    if (type(url) is str):
        __url__ = __NSURL__.URLWithString(url)
        mainthread.run_sync(__openURL__)
    elif (__PySharingHelper__.isURL(url)):
        __url__ = url
        mainthread.run_sync(__openURL__)
    else:
        raise ValueError('url musts be a String or an Objective-C `NSURL`.')

